@@ -11,21 +11,26 @@ class DCERPCSMBTransport:
 		self.smbfile = None
 		
 		self._max_send_frag = None
+
+	def get_session_key(self):
+		return self.target.smb_connection.get_session_key()
 	
-	@red
 	async def connect(self):
 		# TODO: if the smb connection is not set up, we need to set it up
+		try:
+			unc_path = '\\\\%s\\%s%s' % (self.target.smb_connection.target.get_hostname_or_ip(), 'IPC$', self.target.pipe)
+			self.smbfile = SMBFile.from_uncpath(unc_path)
+			_, err = await self.smbfile.open(self.target.smb_connection, 'wp')
+			return True, err
+		except Exception as e:
+			return None, e
 
-		unc_path = '\\\\%s\\%s%s' % (self.target.smb_connection.target.get_hostname_or_ip(), 'IPC$', self.target.pipe)
-		self.smbfile = SMBFile.from_uncpath(unc_path)
-		_, err = await self.smbfile.open(self.target.smb_connection, 'wp')
-		return True, err
-
-	@red
 	async def disconnect(self):
-		await self.smbfile.close()
+		try:
+			await self.smbfile.close()
+		except Exception as e:
+			return None, e
 	
-	@red
 	async def send(self, data, forceWriteAndx = 0, forceRecv = 0):
 		try:
 			if self._max_send_frag:
@@ -48,10 +53,11 @@ class DCERPCSMBTransport:
 		except Exception as e:
 			return None, e
 
-	
-	@red
 	async def recv(self, count): #async def recv(self, forceRecv = 0, count = 0):
-		#print(count)
-		data, err = await self.smbfile.read(count)
-		#print('recv %s' % repr(data))
-		return data, err
\ No newline at end of file
+		try:
+			#print(count)
+			data, err = await self.smbfile.read(count)
+			#print('recv %s' % repr(data))
+			return data, err
+		except Exception as e:
+			return None, e
\ No newline at end of file
