@@ -10,10 +10,9 @@ from aiosmb.examples.smbpathcompleter import SMBPathCompleter
 
 from aiosmb import logger
 from aiosmb._version import __banner__
-from aiosmb.commons.connection.url import SMBConnectionURL
+from aiosmb.commons.connection.factory import SMBConnectionFactory
 from aiosmb.commons.interfaces.machine import SMBMachine
 from aiosmb.commons.interfaces.share import SMBShare
-from aiosmb.commons.utils.decorators import rr, rr_gen, red, red_gen, ef_gen
 from aiosmb.commons.exceptions import SMBException, SMBMachineException
 from aiosmb.dcerpc.v5.rpcrt import DCERPCException
 
@@ -32,19 +31,38 @@ def req_traceback(funct):
 	return wrapper
 
 class SMBClient(aiocmd.PromptToolkitCmd):
-	def __init__(self, url = None, silent = False):
+	def __init__(self, url = None, silent = False, no_dce = False):
 		aiocmd.PromptToolkitCmd.__init__(self, ignore_sigint=False) #Setting this to false, since True doesnt work on windows...
 		self.conn_url = None
 		if url is not None:
-			self.conn_url = SMBConnectionURL(url)
+			self.conn_url = SMBConnectionFactory.from_url(url)
 		self.connection = None
 		self.machine = None
 		self.is_anon = False
 		self.silent = silent
+		self.no_dce = no_dce # diables ANY use of the DCE protocol (eg. share listing) This is useful for new(er) windows servers where they forbid the users to use any form of DCE
 
 		self.shares = {} #name -> share
 		self.__current_share = None
 		self.__current_directory = None
+	
+	async def do_coninfo(self):
+		try:
+			from aiosmb._version import __version__ as smbver
+			from asysocks._version import __version__ as socksver
+			from minikerberos._version import __version__ as kerbver
+			from winacl._version import __version__ as winaclver
+
+			print(self.conn_url)
+			print('AIOSMB: %s' % smbver)
+			print('ASYSOCKS: %s' % socksver)
+			print('MINIKERBEROS: %s' % kerbver)
+			print('WINACL: %s' % winaclver)
+			return True, None
+		
+		except Exception as e:
+			traceback.print_exc()
+			return None, e
 
 	async def do_login(self, url = None):
 		"""Connects to the remote machine"""
@@ -52,12 +70,9 @@ class SMBClient(aiocmd.PromptToolkitCmd):
 			if self.conn_url is None and url is None:
 				print('No url was set, cant do logon')
 			if url is not None:
-				self.conn_url = SMBConnectionURL(url)
+				self.conn_url = SMBConnectionFactory.from_url(url)
 
-			cred = self.conn_url.get_credential()
-			
-			if cred.secret is None and cred.username is None and cred.domain is None:
-				self.is_anon = True			
+			cred = self.conn_url.get_credential()				
 			
 			self.connection  = self.conn_url.get_connection()
 			
@@ -67,6 +82,7 @@ class SMBClient(aiocmd.PromptToolkitCmd):
 			_, err = await self.connection.login()
 			if err is not None:
 				raise err
+			self.is_anon = self.connection.gssapi.is_guest()
 			self.machine = SMBMachine(self.connection)
 			if self.silent is False:
 				print('Login success')
@@ -91,6 +107,10 @@ class SMBClient(aiocmd.PromptToolkitCmd):
 	async def _on_close(self):
 		await self.do_logout()
 
+	async def do_nodce(self):
+		"""Disables automatic share listing on login"""
+		self.no_dce = True
+
 	async def do_shares(self, show = True):
 		"""Lists available shares"""
 		try:
@@ -141,6 +161,26 @@ class SMBClient(aiocmd.PromptToolkitCmd):
 		except Exception as e:
 			traceback.print_exc()
 
+
+	async def do_wsessions(self):
+		"""Lists sessions of connected users"""
+		try:
+			async for sess, err in self.machine.wkstlist_sessions():
+				if err is not None:
+					raise err
+				print("%s" % sess.username)
+		except SMBException as e:
+			logger.debug(traceback.format_exc())
+			print(e.pprint())
+		except SMBMachineException as e:
+			logger.debug(traceback.format_exc())
+			print(str(e))
+		except DCERPCException as e:
+			logger.debug(traceback.format_exc())
+			print(str(e))
+		except Exception as e:
+			traceback.print_exc()
+
 	async def do_domains(self):
 		"""Lists domain"""
 		try:
@@ -237,10 +277,33 @@ class SMBClient(aiocmd.PromptToolkitCmd):
 		except Exception as e:
 			traceback.print_exc()
 
+	async def do_addsidtolocalgroup(self, group_name, sid):
+		"""Add member (by SID) to a local group"""
+		try:
+			result, err = await self.machine.add_sid_to_group('Builtin', group_name, sid)
+			if err is not None:
+				raise err
+			if result:
+				print('Modification OK!')
+			else:
+				print('Something went wrong, status != ok')
+			
+		except SMBException as e:
+			logger.debug(traceback.format_exc())
+			print(e.pprint())
+		except SMBMachineException as e:
+			logger.debug(traceback.format_exc())
+			print(str(e))
+		except DCERPCException as e:
+			logger.debug(traceback.format_exc())
+			print(str(e))
+		except Exception as e:
+			traceback.print_exc()
+
 	async def do_use(self, share_name):
 		"""selects share to be used"""
 		try:
-			if self.is_anon is False:
+			if self.is_anon is False or self.no_dce:
 				#anonymous connection might not have access to IPC$ so we are skipping the check
 				if len(self.shares) == 0:
 					_, err = await self.do_shares(show = False)
@@ -522,10 +585,13 @@ class SMBClient(aiocmd.PromptToolkitCmd):
 			traceback.print_exc()
 			return None, e
 	
-	async def do_servicecmdexec(self, command):
+	async def do_servicecmdexec(self, command, timeout = 1):
 		"""Executes a shell command as a service and returns the result"""
 		try:
 			buffer = b''
+			if timeout is None or timeout == '':
+				timeout = 1
+			timeout = int(timeout)
 			async for data, err in self.machine.service_cmd_exec(command):
 				if err is not None:
 					raise err
@@ -669,6 +735,31 @@ class SMBClient(aiocmd.PromptToolkitCmd):
 			traceback.print_exc()
 			return None, e
 
+	async def do_reglistusers(self):
+		"""Saves a registry hive to a file on remote share"""
+		try:
+			users, err = await self.machine.reg_list_users()
+			if err is not None:
+				raise err
+			for user in users:
+				print(user)
+			return True, None
+		
+		except SMBException as e:
+			logger.debug(traceback.format_exc())
+			print(e.pprint())
+			return None, e
+		except SMBMachineException as e:
+			logger.debug(traceback.format_exc())
+			print(str(e))
+			return None, e
+		except DCERPCException as e:
+			logger.debug(traceback.format_exc())
+			print(str(e))
+			return None, e
+		except Exception as e:
+			traceback.print_exc()
+			return None, e
 
 	async def do_get(self, file_name):
 		"""Download a file from the remote share to the current folder"""
@@ -695,6 +786,7 @@ class SMBClient(aiocmd.PromptToolkitCmd):
 								break
 							outfile.write(data)
 							pbar.update(len(data))
+			
 			return True, None
 		except SMBException as e:
 			logger.debug(traceback.format_exc())
@@ -751,6 +843,8 @@ class SMBClient(aiocmd.PromptToolkitCmd):
 			async for secret, err in self.machine.dcsync(target_users=users):
 				if err is not None:
 					raise err
+				if secret is None:
+					continue
 				print(str(secret))
 			
 			return True, None
@@ -842,7 +936,7 @@ class SMBClient(aiocmd.PromptToolkitCmd):
 	async def do_tasks(self):
 		"""List scheduled tasks """
 		try:
-			async for taskname, err in await self.machine.tasks_list():
+			async for taskname, err in self.machine.tasks_list():
 				if err is not None:
 					raise err
 				print(taskname)
@@ -920,10 +1014,26 @@ class SMBClient(aiocmd.PromptToolkitCmd):
 			return None, e
 
 
-	async def do_taskcmdexec(self, command):
+	async def do_taskcmdexec(self, command, timeout = 1):
 		""" Executes a shell command using the scheduled tasks service"""
 		try:
-			await self.machine.tasks_execute_commands([command])
+			buffer = b''
+			if timeout is None or timeout == '':
+				timeout = 1
+			timeout = int(timeout)
+			async for data, err in self.machine.tasks_cmd_exec(command, timeout):
+				if err is not None:
+					raise err
+				if data is None:
+					break
+				
+				try:
+					print(data.decode())
+				except:
+					print(data)
+			return True, None
+			
+			#await self.machine.tasks_execute_commands([command])
 		except SMBException as e:
 			logger.debug(traceback.format_exc())
 			print(e.pprint())
@@ -990,6 +1100,132 @@ class SMBClient(aiocmd.PromptToolkitCmd):
 			traceback.print_exc()
 			return None, e
 
+	async def do_printerenumdrivers(self):
+		""" Enumerates all shares for all files and folders recursively """
+		try:
+			drivers, err = await self.machine.enum_printer_drivers()
+			if err is not None:
+				raise err
+			for driver in drivers:
+				print(driver)
+			return True, None
+		except SMBException as e:
+			logger.debug(traceback.format_exc())
+			print(e.pprint())
+			return None, e
+		except SMBMachineException as e:
+			logger.debug(traceback.format_exc())
+			print(str(e))
+			return None, e
+		except DCERPCException as e:
+			logger.debug(traceback.format_exc())
+			print(str(e))
+			return None, e
+		except Exception as e:
+			traceback.print_exc()
+			return None, e
+
+	async def do_printnightmare(self, share, driverpath = ''):
+		""" printnightmare bug using the RPRN protocol """
+		try:
+			if len(driverpath) == 0:
+				driverpath = None
+			_, err = await self.machine.printnightmare(share, driverpath)
+			if err is not None:
+				raise err
+			return True, None
+		except SMBException as e:
+			logger.debug(traceback.format_exc())
+			print(e.pprint())
+			return None, e
+		except SMBMachineException as e:
+			logger.debug(traceback.format_exc())
+			print(str(e))
+			return None, e
+		except DCERPCException as e:
+			logger.debug(traceback.format_exc())
+			print(str(e))
+			return None, e
+		except Exception as e:
+			traceback.print_exc()
+			return None, e
+	
+	async def do_parprintnightmare(self, share, driverpath = ''):
+		""" printnightmare bug using the PAR protocol """
+		try:
+			if len(driverpath) == 0:
+				driverpath = None
+			_, err = await self.machine.par_printnightmare(share, driverpath)
+			if err is not None:
+				raise err
+			return True, None
+		except SMBException as e:
+			logger.debug(traceback.format_exc())
+			print(e.pprint())
+			return None, e
+		except SMBMachineException as e:
+			logger.debug(traceback.format_exc())
+			print(str(e))
+			return None, e
+		except DCERPCException as e:
+			logger.debug(traceback.format_exc())
+			print(str(e))
+			return None, e
+		except Exception as e:
+			traceback.print_exc()
+			return None, e
+
+	async def do_backupkeys(self, outfile = None):
+		"""Obtains the DPAPI domain backup keys"""
+		try:
+			backupkeys, err = await self.machine.get_backupkeys()
+			if err is not None:
+				raise err
+			for guid in backupkeys:
+				if outfile is None or len(outfile) == 0:
+					print('GUID: %s' % guid)
+				if 'legacykey' in backupkeys[guid]:
+					if outfile is None or len(outfile) == 0:
+						print('Legacy key: 0x%s' % backupkeys[guid]['legacykey'].hex())
+					else:
+						with open('%s_%s_%s.key' % (outfile, guid, 'legacykey'), 'wb') as f:
+							f.write(backupkeys[guid]['legacykey'])
+				if 'pvk' in backupkeys[guid]:
+					if outfile is None or len(outfile) == 0:
+						print('PVK: %s' % backupkeys[guid]['pvk'].to_bytes().hex())
+					else:
+						with open('%s_%s.pvk' % (outfile, guid), 'wb') as f:
+							f.write(backupkeys[guid]['pvk'].to_bytes())
+				if 'certificate' in backupkeys[guid]:
+					if outfile is None or len(outfile) == 0:
+						print('Certificate: \r\n%s' % backupkeys[guid]['certificate'].hex())
+					else:
+						with open('%s_%s.der' % (outfile, guid), 'wb') as f:
+							f.write(backupkeys[guid]['certificate'])
+				if outfile is None or len(outfile) == 0:
+					print('')
+				else:
+					print('Backupkey saved to disk')
+			return True, None
+		except SMBException as e:
+			logger.debug(traceback.format_exc())
+			print(e.pprint())
+			return None, e
+		except SMBMachineException as e:
+			logger.debug(traceback.format_exc())
+			print(str(e))
+			return None, e
+		except DCERPCException as e:
+			logger.debug(traceback.format_exc())
+			print(str(e))
+			return None, e
+		except Exception as e:
+			traceback.print_exc()
+			return None, e
+
+
+	
+
 async def amain(args):
 	client = SMBClient(args.smb_url, silent = args.silent)
 	if len(args.commands) == 0:
@@ -1004,7 +1240,6 @@ async def amain(args):
 				return
 			
 			cmd = shlex.split(command)
-			#print(cmd)
 			if cmd[0] == 'login':
 				_, err = await client.do_login()
 				if err is not None:
@@ -1018,6 +1253,7 @@ def main():
 	import argparse
 	import platform
 	import logging
+	from asysocks import logger as asylogger
 	
 	parser = argparse.ArgumentParser(description='Interactive SMB client')
 	parser.add_argument('-v', '--verbose', action='count', default=0)
@@ -1037,6 +1273,7 @@ def main():
 		print('setting deepdebug')
 		logger.setLevel(1) #enabling deep debug
 		sockslogger.setLevel(1)
+		asylogger.setLevel(1)
 		asyncio.get_event_loop().set_debug(True)
 		logging.basicConfig(level=logging.DEBUG)
 
